#ifndef __TH5Dataset__
#define __TH5Dataset__

#include <TObject.h>
#include <TNtupleD.h>
#include <TH1F.h>
#include <TH2.h>
#include <TH3.h>
#include <TF1.h>
#include <TVector3.h>
#include <TObjString.h>
#include <TString.h>
#include <TObjArray.h>
#include <TClonesArray.h>
#include <TArrayD.h>
#include <TCanvas.h>
#include <TCut.h>
#include <TGraph.h>

#include <iostream>
#include <string>

//#include "H5Part.h"
#include "TH5Util.h"
#include "TH5ProgressBar.h"

//const Double_t EMPTY(-999.999);

//class H5PartFile;
class TH5ProgressBar;

using namespace std;
using namespace TH5Util;

class TH5Dataset : public TObject
{
 private:
  
  //  H5PartFile* fH5File;      // H5 file
  TString* fH5File;      // H5 file
  TString fFullFilename;    // full filename of H5file
  TString fFilename;        // filename of H5file (without path)
  TString fFilenameNoExt;   // filename of H5file (without path, no extension)
  EH5State fFileState;      // flag for active/loaded...
  Int_t fNStep;             // number of time steps
  Int_t fNStepAttr;         // number of step attributes
  Int_t fNFileAttr;         // number of file attributes
  Int_t fNScalarAttr;       // number of scalar step attributes
  Int_t fNVectorAttr;       // number of vector step attributes

  Int_t fWidthStep;         // format width for step display

  Int_t fNPart0;            // number of particles in step 0
  Int_t fPlotCount;         // plot counter (for plots to remain in memory)

  TObjArray fFileAttr;

  TString fPartVarName[9];
  TString fPartVarUnit[9];

  TObjArray fScalarName;     
  TObjArray fScalarUnit;     
  TObjArray fVectorName;    
  TObjArray fVectorUnit;    

  TArrayD* fSpos;           // spos values 
  TH2D* fScalar;            // 2D histogram to emulate 2D array
  TClonesArray* fVector;    // vector attributes are TClonesArray (TClonesArray of 
                            // TClonesArray of TVector3's...
  TClonesArray* fNtuple;    // array of ntuples (one per step)
  TH2D* fMean;              // array containing variable means (centroid)
  TH3D* fSigma;             // 3D histogram to emulate array of 8x8 sigma matrices 
                            //   (one per step)
  TH2D* fMeanSl;            // array containing variable means (centroid) - for slice
  TH3D* fSigmaSl;           // 3D histogram to emulate array of 8x8 sigma matrices 
                            //   (one per step) - for slice

  TH2D* fMeanSc;            // array containing z projected variable means (centroid)
  TH3D* fSigmaSc;           // 3D histogram to emulate array of 8x8 sigma matrices 
                            //   (one per step) - for screen projected covariances

  TClonesArray* fSliceCut;  // array of TCuts for bunch slice
  TArrayD* fSlicePos;       // slice position for every step

  EH5SlicePosOpt fSlicePosOpt; // option for bunch slice position
  Double_t fSlicePercent;   // percentage of bunch slice
  Int_t fSlicePartPerBin;   // number of particle per bin for slice position determination (peak)

  EH5SliceExtentOpt fSliceExtentOpt;  // option for bunch slice extention
  Double_t  fSliceLength;   // bunch slice length
  EH5SliceUnit  fSliceLengthUnit; // bunch slice length unit
  Double_t fSliceSigma;  // sigma for Gaussian slice weight
  EH5SliceUnit fSliceSigmaUnit;  //unit for gaussian weight calculation

  EH5MemOpt fMemOpt;

  TNtupleD* fActiveNt;       // pointer to currently active Ntuple (keep only one in memory)
  Int_t fActiveStep;        // corresponding active step

  Int_t fSubsampRate;       // subsampling rate  

  TGFrame* fParentFrame;    // pointer to parent frame (if any)

 public:

  TH5Dataset(const char * filename, 
	     const Int_t subsampRate = 1, 
	     const EH5MemOpt memOpt = kLoadAndDump, 
	     TGFrame* parentFrame = NULL); // constructor
  TH5Dataset(const TString& filename, 
	     const Int_t subsampRate = 1, 
	     const EH5MemOpt memOpt = kLoadAndDump,
	     TGFrame* parentFrame = NULL); // alternative constructor
  Int_t Construct(const char * filename); // the real constructor!
  
  ~TH5Dataset(); // Destructor
  
  TGFrame* GetParentMainFrame() { return(fParentFrame); }

  Int_t GetNSteps() const { return(fNStep); }
  Int_t GetNScalarAttr() const { return(fNScalarAttr); }
  Int_t GetNVectorAttr() const { return(fNVectorAttr); }

  TString GetFullFilename() const { return(fFullFilename); }
  TString GetFilename() const { return(fFilename); }
  TString GetFilenameNoExt() const { return(fFilenameNoExt); }

  EH5State GetState() const { return(fFileState); }
  void SetState(const EH5State s) { fFileState=s; }

  Double_t GetSpos(const Int_t step) const ;  
  Int_t GetStep(const Double_t spos) const ;  

  Double_t GetStepVar(const TString* stepVarName, 
		      const Int_t step, 
		      const EH5VarType vtype);
  Double_t GetScalarStepVar(const TString* stepVarName, 
			    const Int_t step) const ;
  TVector3 GetVectorStepVar(const TString* stepVarName, 
			    const Int_t step) const ;
  Double_t GetPartStepVar(const TString* stepVarName, 
			  const Int_t step,
                          const EH5VarType vtype) ;

  TString GetPartVarName(const Int_t index) const ; 
  Int_t GetPartVarIndex(const TString* varName) const ; 
  TString GetScalarStepVarName(const Int_t index) const ;
  TString GetVectorStepVarName(const Int_t index) const ;

  TString GetUnit(const char* vname) const ;
  TString GetUnit(const TString varName) const ;
  TString GetPartUnit(const char* vname) const ;
  TString GetPartUnit(const TString varName) const ;

  TNtupleD* GetNtuple(Int_t step=0) ;
  void DumpNtuple(const Int_t step=0) ;

  Double_t GetPartMean(const TString* varName, 
		       const Int_t step=0, 
		       const EH5VarType vtype=kPart);
  Double_t GetPartCov(const TString* varNameX, 
		      const TString* varNameY, 
		      const Int_t step=0, 
		      const EH5VarType vtype=kPart);
  Double_t GetPartEmit(const TString* varNameX, 
		       const TString* varNameY, 
		       const Int_t step=0, 
		       const EH5VarType vtype=kPart);
  Double_t GetPartPeak(const TString* varName, 
		       const Int_t step, 
		       const Int_t nBin);

  Double_t GetScreenProjectedStepVar(const TString * varName,
				     const Int_t step,
				     const EH5PartVarType pvtype=kMean);
  Double_t GetScreenProjectedStepVar(const TString * varNameX,
				     const TString * varNameY,
				     const Int_t step,
				     const EH5PartVarType pvtype);
  Int_t GetScreenProjectedStepVarArray(const TString * varNameX,
				       const TString * varNameY,
				       Double_t var[], 
				       const EH5PartVarType pvtype);
  Double_t GetGaussSliceStepVar(const TString* varName,
				const Int_t step,
				const EH5PartVarType varType=kMean);
  Double_t GetGaussSliceStepVar(const TString* varNameX,
				const TString* varNameY,
				const Int_t step,
				const EH5PartVarType pvtype);

  void SetSliceOptions(const EH5SlicePosOpt posOpt=kMeanZ, 
			const Int_t partPerBin=100, 
			const EH5SliceExtentOpt extOpt=kLength, 
			const Double_t percent=20, 
			const Double_t length=0.3, 
			const EH5SliceUnit lengthUnit=kmm,
			const Double_t sigma=0., 
			const EH5SliceUnit sigmaUnit=kmm);

  void SetSlicePosOpt(const EH5SlicePosOpt posOpt,
		      const Int_t partPerBin=100);
  void SetSlicePosition(const char* pos,
			const Int_t partPerBin=100);
  void SetSlicePartPerBin(const Int_t partPerBin=100);
  void SetSliceLength(const Double_t length, 
		      const EH5SliceUnit unit=kmm);
  void SetSliceLength(const Double_t length, 
		      const char* unit);
  void SetSliceLengthUnit(const char* unit);
  void SetSliceLengthUnit(const EH5SliceUnit=kmm);
  void SetSliceSigma(const Double_t sigma, 
		     const EH5SliceUnit unit=kmm);
  void SetSliceSigma(const Double_t sigma, 
		     const char* unit);
  void SetSliceSigmaUnit(const char* unit);
  void SetSliceSigmaUnit(const EH5SliceUnit=kmm);
  void SetSlicePercent(const Double_t percent);

  EH5SlicePosOpt GetSlicePosOpt() const { return(fSlicePosOpt); }  
  Double_t GetSlicePercent() const { return(fSlicePercent); }  
  Int_t GetSlicePartPerBin() const { return(fSlicePartPerBin); }  
  EH5SliceExtentOpt GetSliceExtentOpt() const { return(fSliceExtentOpt); }
  Double_t GetSliceLength() const { return(fSliceLength); }
  EH5SliceUnit GetSliceLengthUnit() const { return(fSliceLengthUnit); }
  Double_t GetSliceSigma() const { return(fSliceSigma); }
  EH5SliceUnit GetSliceSigmaUnit() const { return(fSliceSigmaUnit); }

  TCut* GetSliceCut(const Int_t step=0);
  TCut* GetSliceCutPercent(const TString* varName, 
			   const Int_t step=0,
			   const Double_t slicePercent=20.,
			   const EH5SlicePosOpt posOpt=kMeanZ, 
			   const Int_t slicePartPerBin=100);
  TCut* GetSliceCutLength(const TString* varName, 
			  const Int_t step=0,
			  const Double_t sliceLength=0.3,
			  const EH5SliceUnit sliceLengthUnit=kmm,
			  const EH5SlicePosOpt posOpt=kMeanZ,
			  const Int_t slicePartPerBin=100);
  Double_t GetSlicePos(const TString* varName, const Int_t step=0,
                       const EH5SlicePosOpt posOpt=kPeakZ,
                       const Int_t slicePartPerBin=100);
        
  Int_t LoadStep(const Int_t step=0);

  TH1F* Histo1d(const TString* varName, 
		const Int_t step=0, 
		const Double_t min=-1., 
		const Double_t max = 1., 
		const Int_t nBin=30, 
		const TCut cut = "", 
		const Bool_t keep=kFALSE);

  TH1F* Histo1d(const char* varName, 
		const Int_t step=0, 
		const Double_t min=-1., 
		const Double_t max = 1.,  
		const Int_t nBin=30, 
		const TCut cut = "");  // for interactive use 

  TH2F* Histo2d(const TString* varNameX, 
		const TString* varNameY, 
		const Int_t step=0, 
		const Double_t minX=-1., 
		const Double_t maxX = 1., 
		const Double_t minY=-1., 
		const Double_t maxY = 1., 
		const Int_t nBinX=50, 
		const Int_t nBinY=50, 
		const TCut cut = "",
		const Bool_t keep=kFALSE);
  
  TH2F* Histo2d(const char* varNameX, 
		const char* varNameY, 
		const Int_t step=0, 
		const Double_t minX=-1., 
		const Double_t maxX = 1., 
		const Double_t minY=-1., 
		const Double_t maxY = 1., 
		const Int_t nBinX=50, 
		Int_t nBinY=50, 
		const TCut cut = ""); // for interactive use 
  
  TH1F* Scatter(const TString* varNameX, 
		const TString* varNameY, TCanvas* canv, 
		const Int_t step=0, 
		const Double_t minX=-2., 
		const Double_t maxX = 2., 
		const Double_t minY=-2., 
		const Double_t maxY = 2.,
		const TCut cut = ""); 
  
  TH1F* Scatter(const char* varNameX, 
		const char* varNameY, 
		const Int_t step=0, 
		const Double_t minX=-2., 
		const Double_t maxX = 2., 
		const Double_t minY=-2., 
		const Double_t maxY = 2.,
		const TCut cut = ""); // for interactive use 
  
  TH3F* Histo3d(const TString* varNameX, 
		const TString* varNameY, 
		const TString* varNameZ, 
		const Int_t step=0, 
	        const Double_t minX=-1., 
		const Double_t maxX = 1., 
		const Double_t minY=-1., 
		const Double_t maxY = 1., 
		const Double_t minZ=-1., 
		const Double_t maxZ = 1., 
		const Int_t nBinX=30, 
		const Int_t nBinY=30, 
		const Int_t nBinZ=30,
		const TCut cut = "", 
		const Bool_t keep=kFALSE);

  TH3F* Histo3d(const char* varNameX, 
		const char* varNameY, 
		const char* varNameZ, 
		const Int_t step=0, 
	        const Double_t minX=-1., 
		const Double_t maxX = 1., 
		const Double_t minY=-1., 
		const Double_t maxY = 1., 
		const Double_t minZ=-1., 
		const Double_t maxZ = 1., 
		const Int_t nBinX=30, 
		const Int_t nBinY=30, 
		const Int_t nBinZ=30, 
		const TCut cut = ""); // for interactive use 

  Double_t GetMinimum(const TString varName, Int_t step=0);
  Double_t GetMinimum(const char* varName, Int_t step=0);
  Double_t GetMaximum(const TString varName, Int_t step=0);
  Double_t GetMaximum(const char* varName, Int_t step=0);
  Double_t GetGlobalMinimum(const TString varName);
  Double_t GetGlobalMinimum(const char* varName);
  Double_t GetGlobalMaximum(const TString varName); 
  Double_t GetGlobalMaximum(const char* varName); 
  Double_t PlotRange(const TString* varName, 
		     const Int_t step, 
		     Double_t& plotMin, 
		     Double_t& plotMax);
  Double_t Range(const TString* varName, 
		 const Int_t step, 
		 Double_t& min, 
		 Double_t& max);

  Int_t GetStepVarArray(const TString stepVarName, 
			Double_t var[], 
			const EH5VarType vtype);

  TGraph* GetGraphXY(const char* varNameX, 
		     const char* varNameY, 
		     const char* coordY="",
		     Double_t scaleY=1.0);
  TGraph* GetGraphXY(const char* varNameX, 
		     const char* coordX, 
		     const char* varNameY, 
		     const char* coordY,
		     Double_t scaleY=1.0);
  TGraph* GetGraphXY(const char* varNameX, 
		     const char* coordX,
		     const Double_t scaleX,
		     const char* varNameY, 
		     const char* coordY,
		     Double_t scaleY=1.0);

  TGraph* GetGraphXY(const TString vnamex, 
		     const TString crdx, 
		     const TString vnamey, 
		     const TString crdy,
		     const Double_t scaleY=1.0);

  TGraph* GetGraphXY(const TString vnamex, 
		     const TString crdx, 
		     const Double_t scaleX,
		     const TString vnamey, 
		     const TString crdy,
		     const Double_t scaleY=1.0);

  Int_t DecodeString(TString& s);

  TH1F* HistoSliceEmit(const TString* varNameX,
		       const TString* varNameY,
		       const Int_t step,
		       const Bool_t keep=kFALSE);

  TH1F* HistoSliceEmit(const TString* varNameX,
		       const TString* varNameY,
		       const Int_t step,
		       const Double_t sliceLength,
		       const Double_t sliceSpacing=0.,
		       const EH5SliceUnit sliceUnit=kmm,
		       const Double_t gaussSigma=0.,
		       const Bool_t keep=kFALSE);
  
  TH1F* HistoSliceEmit(const char* varNameX,
		       const char* varNameY,
		       const Int_t step,
		       const Double_t sliceLength=0.,
		       const Double_t sliceSpacing=0.,
		       const EH5SliceUnit sliceUnit=kmm,
		       const Double_t gaussSigma=0.);
  
  Double_t GetSliceEmit(const TString* varNameX,
			const TString* varNameY,
			const Int_t step,
			const Double_t s0,
			const Double_t sliceLength,  // in m!
			const Double_t gaussSigma);  // if = 0, uniform weighting

  Double_t ps2mm(const Double_t length_ps, const Int_t step=0);


/////////////////////////////////////////////////////////////////////////
//***********************************************************************
//                      some auxiliary functions (Bo Liu)
//***********************************************************************
/////////////////////////////////////////////////////////////////////////

// keep them for the time being - perhaps will need something from it later!
  Int_t GetSliceVar(const TString * varNameX, const TString* varNameY,
		    const Double_t spos,
		    const Double_t sigma,
		    const EH5SliceUnit sliceUnit,
		    const Double_t spacing,
		    const Bool_t bGaussian,
		    TNtupleD& sliceV);
  Int_t GetSliceVar(const TString* varNameX, const TString* varNameY,
		    const Int_t step,
		    const Double_t sigma,
		    const EH5SliceUnit sliceUnit,
		    const Double_t spacing,
		    const Bool_t bGaussian,
		    TNtupleD& sliceV);
  Double_t GetSliceVarAt(const TString* varNameX, const TString* varNameY,
			 const Int_t step,
			 const Double_t halfL,
			 const Double_t mean,
			 const Bool_t bGaussian,
			 Double_t& sx,
			 Double_t& sy,
			 Double_t& sxx,
			 Double_t& syy,
			 Double_t& sxy);

  Double_t GetSliceEmittance(const TString* varName, 	
			     const Double_t spos, 
			     const Double_t sigma, 
			     const Double_t spacing, 
			     const Bool_t bGaussian,
			     Double_t zpos[], 
			     Double_t emittance[], 
			     Int_t& count);
  Double_t GetSliceEmittance(const TString* varName, 
			     const Int_t step, 
			     const Double_t sigma, 
			     const Double_t spacing, 
			     const Bool_t bGaussian,
			     Double_t zpos[], 
			     Double_t emittance[], 
			     Int_t& count);
  Double_t GetSliceEmittanceAt(const TString* varName, 
			       const Int_t step, 
			       const Double_t sigma, 
			       const Double_t mean, 
			       const Bool_t bGaussian);
  Double_t GetEmittanceAtFixedZ(const TString* varName,
                                Double_t zpos[],
                                Double_t emittance[]);
  Double_t GetEmittanceAtFixedSingleZ(const TString* varName, 
				      Double_t zpos);

  ClassDef(TH5Dataset,1);
    
};

#endif
