#include <TGMsgBox.h>
#include <iostream>

#include <stdarg.h>
#include <stdio.h>

#include "TH5Util.h"

// three messages handlers (with the vararg stuff it is just too painful to 
// do it with a base class...)

void TH5Util::h5Error(TObject* caller, 
		      const char* location, 
		      const Bool_t popUp, 
		      const char * fmt,...)
{
  // pop up a message window
  va_list ap;
  va_start(ap, fmt);

  char buffer[120];
  vsprintf(buffer,fmt,ap);
  va_end(ap);

  if (popUp) {
    TString msg(buffer);
    new TGMsgBox(gClient->GetRoot(), gClient->GetRoot(), 
		 "H5PartROOT Error",msg,kMBIconStop,kMBOk);
  }

  // also issue a command line error for the record:
  caller->Error(location,buffer);

}

void TH5Util::h5Warning(TObject* caller, 
			const char* location, 
			const Bool_t popUp, 
			const char * fmt,...)
{
  // pop up a message window
  va_list ap;
  va_start(ap, fmt);

  char buffer[120];
  vsprintf(buffer,fmt,ap);
  va_end(ap);

  if (popUp) {
    TString msg(buffer);
    new TGMsgBox(gClient->GetRoot(), gClient->GetRoot(), 
		 "H5PartROOT Warning",msg,kMBIconExclamation,kMBOk);
  }

  // also issue a command line error for the record:
  caller->Warning(location,buffer);

}

void TH5Util::h5Info(TObject* caller, 
		     const char* location, 
		     const Bool_t popUp, 
		     const char * fmt,...)
{
  // pop up a message window
  va_list ap;
  va_start(ap, fmt);

  char buffer[120];
  vsprintf(buffer,fmt,ap);
  va_end(ap);

  if (popUp) {
    TString msg(buffer);
    new TGMsgBox(gClient->GetRoot(), gClient->GetRoot(), 
		 "H5PartROOT Warning",msg,kMBIconAsterisk,kMBOk);
  }    

  // also issue a command line error for the record:
  caller->Info(location,buffer);

}

Double_t TH5Util::AddMargin(Double_t& plotMin, Double_t& plotMax, const Double_t margin) {
  // (by default) add 10% at each edge to make a nice plot
  // check for pathological cases

  Double_t range = plotMax-plotMin;
  Double_t mean = 0.5*(plotMax+plotMin);

  if (mean==0.) mean = 1.e-6;
  if (TMath::Abs(plotMin) < 1.e-8 && TMath::Abs(plotMax) < 1.e-8) {
    plotMin -= 1.;
    plotMax += 1.;
  } else if (plotMin*plotMax > 0. && range/mean < 1.e-8) {
    plotMin = plotMin-margin*TMath::Abs(mean);
    plotMax = plotMax+margin*TMath::Abs(mean);
  } else {
    plotMin = plotMin-margin*range;
    plotMax = plotMax+margin*range;
  }
  return(range);
}

TString TH5Util::AddBrackets(TString s) {
  
  TString sb(" [");
  if (s==TString(" ")||s==TString("")) {
    sb = s;
  } else {
    sb += s;
    sb += TString("] ");
  }
  return(sb);
}


